#include "Character.h"
#include "Camera.h"
#include "World.h"

#define TRAVEL_SPEED (1.0f/1100.0f)

Character::Character(void)
{
	Reset();
	m_indicatorTexture = -1;
}

Character::~Character(void)
{
}


void Character::Go(int ms)
{
	Vector3d delta = Vector3d( cos(m_turnangle) * (((float)ms) * TRAVEL_SPEED),
								sin(m_turnangle) * (((float)ms) * TRAVEL_SPEED),
								0.0f);
	
	World::theWorld->ConstrainToIndoors(Position, delta);

	SetLookat();
}

void Character::Strafe(int ms)
{
	Vector3d delta = Vector3d( cos(m_turnangle - HALF_PI) * (((float)ms) * TRAVEL_SPEED),
								sin(m_turnangle - HALF_PI) * (((float)ms) * TRAVEL_SPEED),
								0.0f);
	
	World::theWorld->ConstrainToIndoors(Position, delta);

	SetLookat();
}

void Character::SetLookat()
{
	//find location on spherical envelope
	Lookat.x = cos(m_turnangle) * sin(m_upangle)	+ Position.x;
	Lookat.y = sin(m_turnangle) * sin(m_upangle)	+ Position.y;
	Lookat.z = cos(m_upangle)						+ Position.z;
}

string Character::GetLookDirection()
{
	string thedir("");
	float theangle = (m_turnangle - FOURTH_PI); // adjust so 0 is a bit left
	theangle = -(theangle - (TWICE_PI * floor(theangle / TWICE_PI))); //find fraction part
	if(theangle < 0) theangle += TWICE_PI; //ensure positive

	//find quadrant
	if(theangle < HALF_PI)
		thedir = "East";
	else if(theangle < PI)
		thedir = "South";
	else if(theangle < PI + HALF_PI)
		thedir = "West";
	else
		thedir = "North";

	return thedir;
}

void Character::ChangeLook(float dx, float dy)
{
	m_turnangle += dx; //wraps 360 degrees

	m_upangle -= dy; //artificially stop at convenient spots
	m_upangle = MAX(FOURTH_PI, m_upangle);
	m_upangle = MIN(3 * FOURTH_PI, m_upangle);

	SetLookat();
}

void Character::Reset()
{
	Camera::Reset();

	Position.x = 0.0f;
	Position.y = 0.0f;
	Position.z = 0.1f;

	m_turnangle = 0.0f;
	m_upangle = HALF_PI;

	SetLookat();
}

void Character::InitializeTexture()
{
	if(m_indicatorTexture != -1)
		return;

	//do texture for indicator
	glGenTextures(1, &m_indicatorTexture);

	GLubyte color[] = {128, 255, 255};

	glBindTexture(GL_TEXTURE_2D, m_indicatorTexture);
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);	// Linear Min Filter
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);	// Linear Mag Filter

	glTexImage2D(GL_TEXTURE_2D, 0, 3, 1, 1, 0, GL_RGB, GL_UNSIGNED_BYTE, color);
}

void Character::RenderDirectionIndicator(float height)
{
	InitializeTexture();

	{
		AutoMatrixStack stack;

		glTranslatef(Position.x, Position.y, height);
		glScalef(1.0, 1.0f, 0.2f);
		glRotatef(-90, 1.0, 0, 0);
		glRotatef(DEG(-m_turnangle) + 90.0f, 0, 1.0f, 0);
		glTranslatef(0,0,-GRID_SIZE/6);
		glBindTexture(GL_TEXTURE_2D, m_indicatorTexture);
		glutSolidCone(GRID_SIZE/8, GRID_SIZE/3, 20, 5);
	}
}