#include "Entity3d.h"

Entity3d::Entity3d(void)
{
	rendered_type = RENDERED_NONE;

	Connectivity.ConnectionsValue = 0;

	xpos = ypos = 0.0f;
	width = length = height = 1;

	GeneratedByID = -1;
	m_entityOwner = NULL;

	memset(variables, 0, sizeof(variables));
	//variable = 0;
}

Entity3d::~Entity3d(void)
{
}

void Entity3d::Initialize(int generatorID, int w, int l, int h, EntityRenderedType renderType)
{
	GeneratedByID = generatorID;

	m_bakingCollisions = false;

	width = ABS(w);
	length = ABS(l);
	height = ABS(h);
	rendered_type = renderType;
	Connectivity.ConnectionsValue = 0;
	m_entityOwner = NULL;

	m_collidables.ClearCollidables();

	memset(variables, 0, sizeof(variables));
}

void Entity3d::Render(ConnectionStruct externalConnections, ConnectionStruct adjacentTunnels, int textureGroup, bool shouldBakeCollisions, float *heights)
{
	m_bakingCollisions = shouldBakeCollisions;
	switch(rendered_type)
	{
	case RENDERED_PADDING:
		//nothing.
		break;
	case RENDERED_JUNCTIONBOXES:
		RenderJunctionBoxes(externalConnections, adjacentTunnels, textureGroup, heights);
		break;
	case RENDERED_SHAFT:
		RenderShaft(externalConnections, adjacentTunnels, textureGroup);
		break;

	case RENDERED_DUMMY:
	default:
		RenderDummy();
		break;
	}
}

void Entity3d::RenderDummy()
{
	if(m_bakingCollisions) return;

	{
		AutoMatrixStack stack;

		glTranslatef(xpos, ypos, 0);
		glScalef(width* GRID_SIZE, length* GRID_SIZE, height* GRID_SIZE);
		glutSolidCube(0.9f);
	}
}

void Entity3d::RenderShaft(ConnectionStruct externalConnections, ConnectionStruct adjacentTunnels, int textureGroup)
{
	if(m_bakingCollisions) return;

	{
		AutoMatrixStack stack;
		glTranslatef(xpos, ypos, 0);
		glScalef(width * GRID_SIZE, length* GRID_SIZE, height* GRID_SIZE);
		glutSolidCube(1.0f);
	}
}

//connection width also is the width/length of the central box
//everything else is based off of this. Tweak as needed.
#define CONNECTION_WIDTH		0.35f

//Connection length must adjust accordingly, to fit within the grid square.
#define CONNECTION_LENGTH		((GRID_SIZE - CONNECTION_WIDTH)/2)

//the translation must be performed in terms of CONNECTION_LENGTH.
//Here we translate 1/2 the length of the connection (achieving the edge)
//and then we move it a further 1/2 of the connection width (moving it out of the central box).
//kind of disgusting. But this is the precise translation needed.
#define TRANSLATE_CONNECTION	(0.5f + ((CONNECTION_WIDTH/2) / CONNECTION_LENGTH))


	//unsigned north	: 1; //u =  1
	//unsigned east		: 1; //v = -1
	//unsigned south	: 1; //u =  1
	//unsigned west		: 1; //v = -1
const int dir_iter[] = {0, 1, 0, -1}; //for x, y is +1 offset

void Entity3d::RenderJunctionBoxes(ConnectionStruct externalConnections, ConnectionStruct adjacentTunnels, int textureGroup, float *heights)
{
	//position in correct location
	AutoMatrixStack stack;

	//use variable for height
	glScalef(1.0f, 1.0f, variables[0]);
	glTranslatef(xpos, ypos, 0.5);

	//CLARMP!
	variables[1] = MAX(0.0f, variables[1]);
	variables[1] = MIN(1.0f, variables[1]);

	//compute bevel info
	BevelInfo bevel;
	BevelInfo *thesame = &bevel;
	*((unsigned char*)(thesame)) = 0xFF * variables[1];

	//compute unit of bevel
	float max_bevel = CONNECTION_LENGTH / 2.0f;
	float bevel_unit = max_bevel / 3.0f;

	//big box
	if(Connectivity.ConnectionsValue == 0)
	{
		AutoMatrixStack stack;
			DrawTopAndBottom(textureGroup);

			for(int i = 0; i < 4; i++)
			{
				//iterate four directions.
				int delx = dir_iter[i];
				int dely = dir_iter[(i + 1) % 4];

				if(!externalConnections.GetByIndex(i))
					DrawSideFace((Direction)i, textureGroup);
				else if(adjacentTunnels.GetByIndex(i))
				{
					//draw two extra walls
					AutoMatrixStack stack;

					glScalef(1.0f + (ABS(dely) * (CONNECTION_LENGTH - 1.0f)), 1.0f + (ABS(delx) * (CONNECTION_LENGTH - 1.0f)), 1.0f);

					glTranslatef(TRANSLATE_CONNECTION * dely, TRANSLATE_CONNECTION * delx, 0);
					
					if((variables[0] > heights[i]))
					{
						//scale down extra wall sizes in order to prevent overlap with the height-fix below
						glTranslatef(0,0, -0.5 + heights[i]/(2*variables[0]));
						glScalef(1.0f, 1.0f, heights[i]/variables[0]);
					}
					DrawSideFace((Direction)i, textureGroup);

					glTranslatef(2*TRANSLATE_CONNECTION*(-dely), 2*TRANSLATE_CONNECTION*(-delx), 0);
					DrawSideFace((Direction)i, textureGroup);
				}

				if(externalConnections.GetByIndex(i) && (variables[0] > heights[i]))
				{
					//this cell is taller than the next one over.
					AutoMatrixStack stack;
					float difference = variables[0] - heights[i];
					glTranslatef(0,0, heights[i]/(2*variables[0]));
					glScalef(1.0f, 1.0f, difference/variables[0]);
					DrawSideFace((Direction)i, textureGroup);
				}
			}
	}
	else
	{
		{
			//central box
			AutoMatrixStack stack;

			glScalef(CONNECTION_WIDTH, CONNECTION_WIDTH, 1.0f);

			DrawTopAndBottom(textureGroup);

			for(int i = 0; i < 4; i++)
			{
				if(!Connectivity.Connections.GetByIndex(i))
					DrawSideFace((Direction)i, textureGroup);
			}
		}


		for(int i = 0; i < 4; i++)
		{

			//this one goes by CORNERS, not by directions.
			if(Connectivity.Connections.GetByIndex(i) && Connectivity.Connections.GetByIndex(i+1) && (bevel.GetByIndex(i) != 0))
			{
				//do beveled edge
				float bevelDis = bevel_unit * bevel.GetByIndex(i);
				float connection_length = (CONNECTION_LENGTH - bevelDis);
				float translate_connection = (0.5f + ((bevelDis + CONNECTION_WIDTH/2) / connection_length));

				int delx = dir_iter[i];
				int dely = dir_iter[(i + 1) % 4];

				//regardless, we must cap the end, and draw ceiling/floor for this quadrant.
				{
					AutoMatrixStack stack;

					glScalef(CONNECTION_LENGTH * ABS(delx) + CONNECTION_WIDTH * ABS(dely), CONNECTION_LENGTH * ABS(dely) + CONNECTION_WIDTH * ABS(delx), 1.0f);
					glTranslatef(TRANSLATE_CONNECTION*delx, TRANSLATE_CONNECTION*dely, 0);
					DrawTopAndBottom(textureGroup);
					if(!externalConnections.GetByIndex(i))
						DrawSideFace((Direction)i, textureGroup);

					if(externalConnections.GetByIndex(i) && (variables[0] > heights[i]))
					{
						//this cell is taller than the next one over.
						AutoMatrixStack stack;
						float difference = variables[0] - heights[i];
						glTranslatef(0,0, heights[i]/(2*variables[0]));
						glScalef(1.0f, 1.0f, difference/variables[0]);
						DrawSideFace((Direction)i, textureGroup);
					}
				}


				for(int k = 0; k < 2; k++)
				{
					if(textureGroup == k+1)
					{
						AutoMatrixStack stack;
							//Triangle patch
							glRotatef( -i*90 + 45, 0, 0, 1.0f);
							glTranslatef(SQRT_2 * CONNECTION_WIDTH/2, 0, 0);
							
							glRotatef( 180*(k+1), 1.0f, 0, 0);
							glTranslatef(0,0,-0.5f);
							glScalef(SQRT_2 * bevelDis / 2, SQRT_2 * bevelDis, 1.0f);
							glRotatef(180, 1.0f, 0, 0);

							//sooooo ug-layyy 
							glBegin(GL_TRIANGLES);
								glTexCoord2f(0.0f, 0.0f); glVertex3f(0, 0, 0.0f);
								glTexCoord2f(1.0f, 0.5f); glVertex3f(1.0f, 0.5f, 0.0f);
								glTexCoord2f(1.0f, -0.5f); glVertex3f(1.0f, -0.5f, 0.0f);
							glEnd();
					}
				}

				//we perform raw DoQuads here - so check that we're in the right mood/mode
				if(textureGroup == 0 || m_bakingCollisions)
				{
					{
						AutoMatrixStack stack;

						glScalef(connection_length * ABS(delx) + CONNECTION_WIDTH * ABS(dely), connection_length * ABS(dely) + CONNECTION_WIDTH * ABS(delx), 1.0f);
						glTranslatef(translate_connection*delx, translate_connection*dely, 0);
						DrawSideFace((Direction) (((i + 1) + 4) % 4), textureGroup);
					}

					delx = dir_iter[(i + 1) % 4];
					dely = dir_iter[(i + 2) % 4];

					{
						AutoMatrixStack stack;
						glScalef(connection_length * ABS(delx) + CONNECTION_WIDTH * ABS(dely), connection_length * ABS(dely) + CONNECTION_WIDTH * ABS(delx), 1.0f);
						glTranslatef(translate_connection*delx, translate_connection*dely, 0);
						DrawSideFace((Direction) (((i) + 4) % 4), textureGroup);
					}

					{
						AutoMatrixStack stack;

						//draw beveled edge! Draw eastward and then rotate into position.
						glRotatef( -i*90 + 45, 0, 0, 1.0f);
						glScalef(1.0f, bevelDis * SQRT_2, 1.0f);

						glTranslatef(SQRT_2 * CONNECTION_WIDTH/2 + (bevelDis * SQRT_2) / 2, 0, 0);

						glRotatef(-90, 0, 1.0f, 0); 
						glRotatef(180, 1.0f, 0, 0);
						DoQuad();
						//DrawSideFace(EAST, textureGroup);
					}
				}

			}
			else // no bevel
			{
				if(Connectivity.Connections.GetByIndex(i)) //first dir
				{
					//do first dir
					int delx = dir_iter[i];
					int dely = dir_iter[(i + 1) % 4];

					{
						AutoMatrixStack stack;

						glScalef(CONNECTION_LENGTH * ABS(delx) + CONNECTION_WIDTH * ABS(dely), CONNECTION_LENGTH * ABS(dely) + CONNECTION_WIDTH * ABS(delx), 1.0f);
						glTranslatef(TRANSLATE_CONNECTION*delx, TRANSLATE_CONNECTION*dely, 0);
						DrawTopAndBottom(textureGroup);
						DrawSideFace((Direction) (((i + 1) + 4) % 4), textureGroup);
						if(!externalConnections.GetByIndex(i))
							DrawSideFace((Direction)i, textureGroup);

						if(externalConnections.GetByIndex(i) && (variables[0] > heights[i]))
						{
							//this cell is taller than the next one over.
							AutoMatrixStack stack;
							float difference = variables[0] - heights[i];
							glTranslatef(0,0, heights[i]/(2*variables[0]));
							glScalef(1.0f, 1.0f, difference/variables[0]);
							DrawSideFace((Direction)i, textureGroup);
						}
					}
				}

				if(Connectivity.Connections.GetByIndex(i+1)) //second dir
				{
					int delx = dir_iter[(i+1) % 4];
					int dely = dir_iter[(i + 2) % 4];
					{
						AutoMatrixStack stack;

						glScalef(CONNECTION_LENGTH * ABS(delx) + CONNECTION_WIDTH * ABS(dely), CONNECTION_LENGTH * ABS(dely) + CONNECTION_WIDTH * ABS(delx), 1.0f);
						glTranslatef(TRANSLATE_CONNECTION*delx, TRANSLATE_CONNECTION*dely, 0);

						//already capped above.
						//already drew top/bottom above.
						//we're just here to finish drawing the faces:
						DrawSideFace((Direction) (((i) + 4) % 4), textureGroup);
					}
				}
			} //end if should draw bevel
		}
	}
}

void Entity3d::DrawFace(float yangle, float zangle)
{
	AutoMatrixStack stack;
	
	glRotatef(yangle, 0, 1.0f, 0); 
	glRotatef(zangle, 1.0f, 0, 0);

	glTranslatef(0,0,+0.5);

	DoQuad();

}

void Entity3d::DoQuad()
{
	#define HALF_UNIT  (GRID_SIZE / 2)

	if(!m_bakingCollisions)
	{
		glBegin(GL_QUADS);
			glTexCoord2f(-HALF_UNIT,	HALF_UNIT);		glVertex3f(-HALF_UNIT, HALF_UNIT, 0.0f);
			glTexCoord2f(HALF_UNIT,		HALF_UNIT);		glVertex3f(HALF_UNIT, HALF_UNIT, 0.0f);
			glTexCoord2f(HALF_UNIT,		-HALF_UNIT);	glVertex3f(HALF_UNIT, -HALF_UNIT, 0.0f);
			glTexCoord2f(-HALF_UNIT,	-HALF_UNIT);	glVertex3f(-HALF_UNIT, -HALF_UNIT, 0.0f);
		glEnd();
	}
	else
	{
		//get current matrix
		float modelview[16];
		glGetFloatv(GL_MODELVIEW_MATRIX, modelview);
		m_collidables.AddCollisionPlane(modelview,
				Vector3d(-HALF_UNIT, HALF_UNIT, 0.0f),			
				Vector3d(HALF_UNIT, HALF_UNIT, 0.0f),
				Vector3d(HALF_UNIT, -HALF_UNIT, 0.0f),
				Vector3d(-HALF_UNIT, -HALF_UNIT, 0.0f));
	}
}

void Entity3d::DrawTopAndBottom(int textureGroup)
{
	//only collide with walls for now.
	if(textureGroup == 1)// || m_bakingCollisions)
	{
		DrawFace(0);
	}

	if(textureGroup == 2)// || m_bakingCollisions)
	{
		DrawFace(180);
	}
}

void Entity3d::DrawSideFace(Direction dir, int textureGroup)
{
	if(textureGroup == 0 || m_bakingCollisions)
	{
		switch(dir)
		{
		case WEST:
			DrawFace(90, 180);
			break;
		case EAST:
			DrawFace(-90, 180);
			break;
		case NORTH:
			DrawFace(-90, -90);
			break;
		case SOUTH:
			DrawFace(90, 90);
			break;
		}
	}
}

void Entity3d::GetCollidables(CollidableBatch *batch)
{
	m_collidables.AddCollidablesToBatch(batch);
}